#
# OpenOfficeDoc - OpenOffice.org Document class
#
# (c) Copyright 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#
# Released unter the terms of the Mozilla Public License (MPL) v1.1
#
# Please see README.txt for further information
#

"""
OpenOffice.org Document class

$Id: OpenOfficeDoc.py,v 1.1 2002/11/15 22:09:55 philipp Exp $
"""

import sys, os
from types import StringType
from fnmatch import fnmatch
from zipfile import ZipFile

import xml.dom.minidom

from IOfficeDocument import IOfficeDocument


oo_extensions = {
    "text":         "sxw",
    "spreadsheet":  "sxc",
    "presentation": "sxi",
    "drawing":      "sxd"
    }

oo_mimetypes = {
    "text":        "application/vnd.sun.xml.writer",
    "spreadsheet": "application/vnd.sun.xml.calc",
    "drawing":     "application/vnd.sun.xml.impress"
    }


class OpenOfficeDoc:
    """
    OpenOffice.org Document class
    """

    __implements__ = IOfficeDocument

    def __init__( self, fileobj ):
        self._zip = ZipFile( fileobj )


    def getDocType( self ):
        """
        Return the type of document (writer, spreadsheet, etc.)
        """
        content_xml = self.getFile( "content.xml" )
        dom = xml.dom.minidom.parseString( content_xml )
        document_content = dom.getElementsByTagName( "office:document-content" )[0]
        return str(document_content.getAttribute("office:class"))


    def getMetaData( self ):
        """
        Return the meta data
        """
        meta = {}
        meta_xml = self.getFile( "meta.xml" )
        dom = xml.dom.minidom.parseString( meta_xml )

        # Dublin Core properties
        for info in ( "title", "subject", "language" ):
            elements = dom.getElementsByTagName( "dc:"+info )
            meta[info] = ""
            if len(elements) and len(elements[0].childNodes):
                meta[info] = elements[0].childNodes[0].data

        # other meta data
        for info in ( "creation-date", "editing-cycles", "editing-duration" ):
            elements = dom.getElementsByTagName( "meta:"+info )
            meta[info] = ""
            if len(elements) and len(elements[0].childNodes):
                meta[info] = elements[0].childNodes[0].data

        # keywords
        keywords = []
        for keyword in dom.getElementsByTagName( "meta:keyword" ):
            keywords.append( keyword.childNodes[0].data )
        meta["keywords"] = " ".join( keywords )

        # user defined
        user_defined = dom.getElementsByTagName( "meta:user-defined" )
        user_defined_list = []
        for element in user_defined:
            value = ""
            if len(element.childNodes):
                value = element.childNodes[0].data
            user_defined_list.append( { "name": element.getAttribute("meta:name"),
                                        "value": value   } )
        meta["user-defined"] = user_defined_list
        
        # statistics
        statistics = dom.getElementsByTagName( "meta:document-statistic" )[0]
        meta["statistics"] = {
            "table-count":     statistics.getAttribute("meta:table-count"),
            "image-count":     statistics.getAttribute("meta:image-count"),
            "object-count":    statistics.getAttribute("meta:object-count"),
            "page-count":      statistics.getAttribute("meta:page-count"),
            "paragraph-count": statistics.getAttribute("meta:paragraph-count"),
            "word-count":      statistics.getAttribute("meta:word-count"),
            "character-count": statistics.getAttribute("meta:character-count"),
            "cell-count":      statistics.getAttribute("meta:cell-count")
            }
        
        return meta


    def getFile( self, name ):
        """
        Return a file from the archive as data stream
        """
        return self._zip.read( name )


    def getFiles( self, files ):
        """
        Returns a dictionary with filenames as keys and the file contents as values
        """
        if type( files ) == StringType:
            filelist = self._zip.namelist()
            filenames = [n for n in filelist if fnmatch( n, files )]
            
        filedict = {}
        for filename in filenames:
            filedict[filename] = self.getFile( filename )
        return filedict


    def getPictures( self ):
        return self.getFiles( "Pictures/*" )


    def getXMLFiles( self ):
        return self.getFiles( "*xml" )
