#
# ZooDocument - Zope OpenOffice Document
#
# (c) Copyright 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#
# Released unter the terms of the Mozilla Public License (MPL) v1.1
#
# Please see README.txt for further information
#

"""
ZooDocument - Zope OpenOffice Document

$Id: ZooDocument.py,v 1.7 2002/11/16 10:41:43 philipp Exp $
"""

import os

import Globals
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager
from OFS.Image import File, Image, cookId
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.XMLTransform.ProcessorChooser import ProcessorChooser
from OpenOfficeDoc import OpenOfficeDoc, oo_extensions, oo_mimetypes


VIEW_PERM = "View"
CHANGE_PERM = "Change OpenOffice.org Documents"
MANAGE_PERM = "Manage OpenOffice.org Documents"


class ZooDocument( ZopePageTemplate, ObjectManager ):
    """
    ZooDocument
    """

    meta_type = "OpenOffice.org Document"
    icon = "www/zoodoc.png"
    manage_options = (
        {'label':'Document', 'action':'oo_document' },
        ZopePageTemplate.manage_options[0],
        {'label':'Pictures', 'action':'oo_pictures' }, ) + ZopePageTemplate.manage_options[1:]
    _properties = ZopePageTemplate._properties + ( { 'id': 'xslt_processor', 'type': 'selection',
                                                     'mode': 'w', 'select_variable': 'availableProcessors' }, )


    security = ClassSecurityInfo()

    def __init__( self, id, title, docfile="", xslt_processor="" ):
        self.id = id
        self.title = title
        self.xslt_processor = xslt_processor

        if type( docfile ) is not type(""):
            docfile = docfile.read()
            
        self._document = docfile
        
    def manage_afterAdd( self, item, container ):
        # this has to be done here (not in __init__) because we need acquisition
        # for the patcher
        self.processDocument()

    def _setId( self, id ):
        self.id = id
        # this has to be done after every rename because of the picture links
        self.doHtml()


    security.declarePrivate( "processDocument" )
    def processDocument( self ):
        """
        Process the OpenOffice.org document
        """
        doc = OpenOfficeDoc( StringIO( self._document ) )

        # extract the metadata
        self._doctype = doc.getDocType()
        self._meta = doc.getMetaData()

        # extract pictures and add them to the ObjectManager
        self.manage_delObjects( self.objectIds() )
        for filename, data in doc.getPictures().items():
            filename = os.path.split( filename )[1]
            image = Image( filename, "", data )
            self._setObject( filename, image )

        # do the HTML conversion
        self.doHtml()


    security.declarePrivate( "doHtml" )
    def doHtml( self ):
        """
        Processes the XML content and save the resulting HTML
        """
        doc = OpenOfficeDoc( StringIO( self._document ) )
        xmlContents = doc.getFile( "content.xml" )

        # remove the doctype because it's not available as a file and
        # we don't want XML parsers to check the validity here
        doctype = '<!DOCTYPE office:document-content PUBLIC "-//OpenOffice.org//DTD OfficeDocument 1.0//EN" "office.dtd">'
        xmlContents = xmlContents.replace( doctype, "" )
        
        xslfilename = os.path.join( os.path.dirname(__file__), "ooo2html.xsl" )
        xslfile = open( xslfilename )
        xsltContents = xslfile.read()
        xslfile.close()
        
        # get a processor object
        proc_chooser = ProcessorChooser()
        available_procs = proc_chooser.processors()
        if not len( available_procs ):
            return ( "", "" )
        proc = proc_chooser.processorObject( self.xslt_processor )
        
        html = proc.transform( xmlContents, "", xsltContents, "" )
        self.xslt_processor = proc.name

        # add the id in front of all picture links
        html = html.replace( "#Pictures/", self.getId()+"/" )

        # this allows us to install a script or something similar that
        # patches the HTML source before it is written to the Page Template
        try:
            patcher = getattr( self, "zooPatch" )
            html = patcher( html )
        except:
            pass
        
        self.pt_edit( html, "text/html" )


    security.declareProtected( VIEW_PERM, "getDocType" )
    def getDocType( self ):
        return self._doctype

    security.declareProtected( VIEW_PERM, "getMetaData" )
    def getMetaData( self ):
        return self._meta

    security.declarePublic( "availableProcessors" )
    def availableProcessors( self ):
        proc_chooser = ProcessorChooser()
        return proc_chooser.processors()


    ## ObjectManager methods ##

    def objectIds( self, *args ):
        return ObjectManager.objectIds( self, *args )

    def objectValues( self, *args ):
        return ObjectManager.objectValues( self, *args )

    def objectItems( self, *args ):
        return ObjectManager.objectItems( self, *args )

    def objectMap( self ):
        return ObjectManager.objectMap( self )


    ## Methods for returning/editing the document ##

    security.declareProtected( VIEW_PERM, "get_size" )
    def get_size( self ):
        return len( self._document )


    ## Zope Management Interface (ZMI) methods ##

    security.declareProtected( MANAGE_PERM, "oo_document" )
    oo_document = PageTemplateFile( "www/oo_document.pt", globals(),
                                    __name__="oo_document" )

    security.declareProtected( MANAGE_PERM, "oo_pictures" )
    oo_pictures = manage_main = PageTemplateFile( "www/oo_pictures.pt",
                                                  globals(),
                                                  __name__="oo_pictures" )
    

    security.declareProtected( CHANGE_PERM, "oo_download" )
    def oo_download( self, REQUEST, RESPONSE ):
        """
        Download the document for editing
        """
        splitted_filename = self.getId().split(".")
        ext = oo_extensions[self.getDocType()]

        if splitted_filename[-1] != ext:
            if len(splitted_filename) > 1:
                splitted_filename[-1] = ext
            else:
                splitted_filename.append( ext )
        filename = ".".join( splitted_filename )
        
        RESPONSE.setHeader( 'Content-type', oo_mimetypes[self.getDocType()] )
        RESPONSE.setHeader( 'Content-Disposition', 'inline;filename=%s' % filename )
        return self._document


    security.declareProtected( CHANGE_PERM, "oo_upload" )
    def oo_upload( self, docfile="", REQUEST=None ):
        """
        Replace the contents of the document with the text in file.
        """

        if self.wl_isLocked():
            raise ResourceLockedError, ( "This document has been locked via WebDAV." )

        if type( docfile ) is not type(""):
            docfile = docfile.read()
        self._document = docfile
        self.processDocument()

        if REQUEST is not None:
            message = "OpenOffice.org document uploaded."
            return self.oo_document( self, REQUEST, manage_tabs_message=message )


    security.declareProtected( CHANGE_PERM, "oo_download" )
    def oo_convert( self, REQUEST ):
        """
        Convert the document
        """
        self.processDocument()
        if REQUEST is not None:
            message = "OpenOffice.org document converted."
            return self.oo_document( self, REQUEST, manage_tabs_message=message )


    def om_icons( self ):
        """
        Return a list of icon URLs to be displayed by an ObjectManager
        """
        icons = ( {'path': 'misc_/ZooDocument/zoodoc.png',
                   'alt':   self.meta_type,
                   'title': self.meta_type },)

        if not getattr( self, "_v_cooked", 1 ):
            self._cook()
        if self._v_errors:
            icons = icons + ({'path': 'misc_/PageTemplates/exclamation.gif',
                              'alt':  'Error',
                              'title': 'This template has an error'},)
        return icons

        
    ## Protocol handlers ##

    security.declareProtected( CHANGE_PERM, "PUT" )
    def PUT( self, REQUEST, RESPONSE ):
        """
        Handle HTTP PUT requests.
        """
        self.dav__init( REQUEST, RESPONSE )
        self.dav__simpleifhandler( REQUEST, RESPONSE, refresh=1 )
        body = REQUEST.get( 'BODY', '' )
        self.oo_upload( body )
        RESPONSE.setStatus( 204 )
        return RESPONSE


    security.declareProtected( VIEW_PERM, "manage_FTPget" )
    def manage_FTPget( self ):
        """
        Get source for FTP download
        """
        return self._document


    security.declareProtected( MANAGE_PERM, "repair" )
    def repair( self ):
        self.processDocument()


Globals.InitializeClass( ZooDocument )


### object constructors ###

addZooDocumentForm = PageTemplateFile( "www/addZooDoc.pt", globals(), __name__="addZooDocumentform" )


def addZooDocument( self, id="", title="", docfile=None, xslt_processor="", REQUEST=None ):
    """
    Create a ZooDocument instance to an ObjectManager
    """

    id, title = cookId( id, title, docfile )
    obj_instance = ZooDocument( id, title, docfile, xslt_processor )
    self.Destination()._setObject( id, obj_instance )

    if REQUEST is None:
        return

    try:
        url = self.DestinationURL()
    except:
        url = REQUEST['URL1']
    if REQUEST['submit'] == " Add and Edit ":
        url = "%s/%s" % ( url, id )
        REQUEST.RESPONSE.redirect( url + '/oo_document' )
    else:
        REQUEST.RESPONSE.redirect( url + "/manage_main" )
